import { createSlice } from '@reduxjs/toolkit'

import * as types from '../const'

const initialState = {
  friends: {} as Record<string, number>,
  user: {},
}

const setStars = (state, action) => ({
  ...state,
  friends: {
    ...state.friends,
    [action.payload.id]: action.payload.value,
  },
})

const changeStars = (value) => (state, action) => ({
  ...state,
  friends: {
    ...state.friends,
    [action.payload.id]: state.friends[action.payload.id] + value,
  },
})

const createReducer = <State>(initialState: State, reducers) => (state = initialState, action): State => {
  const reducer = reducers[action.type]
  if (!reducer) return state
  return reducer(state, action)
}

export const starsReducer = createReducer(initialState, {
  [types.SET_STARS]: setStars,
  [types.INCREMENT_STARS]: changeStars(1),
  [types.DECREMENT_STARS]: changeStars(-1),
})

const slice = createSlice({
  name: 'stars',
  initialState: {},
  reducers: {
    setStars(state, action) {
      state[action.payload.id] = action.payload.value
    },
    changeStars,
  },

})
