import * as types from '../const'

const createReducer = <State>(initialState: State, reducers) => (state = initialState, action): State => {
  const reducer = reducers[action.type]
  if (!reducer) return state
  return reducer(state, action)
}

export enum Statuses {
  IDLE = 'IDLE',
  FETCHING = 'FETCHING',
  SUCCESS = 'SUCCESS',
  ERROR = 'ERROR',
}

const initialState = {
  data: null,
  status: Statuses.IDLE,
  error: null as null | string,
}

const startFetching = (state) => ({
  ...state,
  error: null,
  status: Statuses.FETCHING,
})

const fetchSuccess = (state, action) => ({
  ...state,
  error: null,
  data: action.payload,
  status: Statuses.SUCCESS,
})

const fetchError = (state, action) => ({
  ...state,
  error: action.payload,
  status: Statuses.ERROR,
})

const reset = () => ({
  ...initialState
})

export const reducer = createReducer(initialState, {
  [types.USER_FETCH]: startFetching,
  [types.USER_FETCH_SUCCESS]: fetchSuccess,
  [types.USER_FETCH_ERROR]: fetchError,
  [types.USER_FETCH_RESET]: reset,
})